#ifndef PV_TYPES_H
#define PV_TYPES_H

#include <ptxlist.h>

#include <jv_types.h>
#include <libpv.h>

#define LEVEL_ALL	7
#define LEVEL_DEBUG	6
#define LEVEL_INFO	5
#define LEVEL_NOTICE	4
#define LEVEL_WARN	3
#define LEVEL_ERROR	2
#define LEVEL_CRIT	1

#ifdef DEBUG
#define dprintf(_level, _fmt, _arg...)			\
do {							\
	if (_libpv_debuglevel >= _level) {		\
		fprintf(stdout, _fmt, ##_arg);		\
		fflush(stdout);				\
	}						\
} while (0)
#else
#define dprintf(_level, _fmt, _arg...)			\
do { } while (0)
#endif

extern int _libpv_debuglevel;


#ifndef MIN
#define	MIN(a,b) (((a)<(b))?(a):(b))
#endif

#ifndef MAX
#define	MAX(a,b) (((a)>(b))?(a):(b))
#endif


enum pv_type {
	pv_type_undef = 0,

	pv_type_int8,
	pv_type_uint8,

	pv_type_int16,
	pv_type_uint16,

	pv_type_int32,
	pv_type_uint32,

	pv_type_int64,
	pv_type_uint64,

	pv_type_string,
};

/* enum pv_unit { */
/* 	pv_unit_none = 0, */
/* }; */


/* enum pv_scale { */
/* 	pv_scale_none = 0, */
/* 	pv_scale_giga, */
/* 	pv_scale_mega, */
/* 	pv_scale_kilo, */
/* 	pv_scale_milli, */
/* 	pv_scale_micro, */
/* 	pv_scale_nano, */
/* 	pv_scale_pico, */
/* 	pv_scale_custom, */
/* }; */

struct pv_backing_store;
typedef struct pv_backing_store pv_backing_store_t;


struct pv_backing_store {
	int	(*init)		(pv_backing_store_t	*,
				 const char		*filename,
				 size_t			size);

	int	(*exit)		(pv_backing_store_t	*);

	int	(*read)		(pv_backing_store_t	*,
				 off_t			,
				 size_t			);

	int	(*write)	(pv_backing_store_t	*,
				 off_t			,
				 size_t			);

	int	shmid;

	void	*map;		/*
				 * set to NULL for not mmappable backing stores, 
				 * the user will be prompted for a shm segment
				 */
	size_t	size;

	void	*priv;
};


union pv_dflt {
	int8_t		i8;
	uint8_t		u8;

	int16_t		i16;
	uint16_t	u16;

	int32_t		i32;
	uint32_t	u32;

	uint64_t	u64;
	int64_t		i64;
	
	char		*string;
};


union pv_data {
	int8_t		i8[0];
	uint8_t		u8[0];

	int16_t		i16[0];
	uint16_t	u16[0];

	int32_t		i32[0];
	uint32_t	u32[0];

	uint64_t	u64[0];
	int64_t		i64[0];
	
	char		string[0];

	void		*ptr;
};


struct jv_server {
	/* JVisu */
	int	jv_port;			/* TCP port for JVisu; default=2202 */
	struct jvisu_sock
		jv_active_sock[MAX_CONNECTIONS + 1];
	int	jv_connections; 
	fd_set	jv_read_fdset;
	fd_set	jv_write_fdset;
	ptxlist	*jv_varlist; 
	int	jv_registeredvars;		/* FIXME: replace */
	struct timeval
		jv_tact;

	/* more - JVisu */
	pthread_t
		jvsocket_thread;
	int	jvisu_stop;			/* set 1 to stop JVisu server       */
};


struct pv_engine {
	int			pv_store_size;
	int			pv_sem;		/* SysV semaphore                   */
	int			pv_sem_key;

	pv_backing_store_t	store;
	
	/* PV Data */
	ptxlist			*pv_list;	/* list of process variables        */

	/* JVisu */
	struct jv_server	*jv_server;
};


struct pv {
	char 			*id;		/* id of var */
	char			*name;		/* name of var "description" */
						/* FIXME: make this a language index array... (rsc) */ 

	enum pv_type 		type;
	off_t			offset;
	union pv_dflt		dflt;

	size_t			size;
};

#endif	/* !PV_TYPES_H */
